/*
 * Copyright (C) 2010-2013 jeanfi@gmail.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301 USA
 */

#include <stdio.h>
#include <string.h>

#include <json/json.h>
#include <gtk/gtk.h>

#include "http.h"
#include "ttrss.h"
#include "url.h"

static char *session_id;
static char *session_url;

static struct json_object *create_op(const char *op)
{
	struct json_object *j;

	j = json_object_new_object();
	json_object_object_add(j, "op", json_object_new_string(op));

	if (session_id && strcmp(op, "login"))
		json_object_object_add(j,
				       "sid",
				       json_object_new_string(session_id));

	return j;
}

void ttrss_login(const char *url, const char *user, const char *password)
{
	struct json_object *content, *rp, *error, *sid, *rq;
	char *tmp;

	if (session_url)
		free(session_url);

	tmp = url_normalize(url);
	session_url = malloc(strlen(tmp) + strlen("/api/") + 1);
	strcpy(session_url, tmp);
	strcat(session_url, "/api/");
	free(tmp);

	rq = create_op("login");
	json_object_object_add(rq, "user", json_object_new_string(user));
	json_object_object_add(rq,
			       "password",
			       json_object_new_string(password));

	rp = post_json_object(session_url, rq);
	json_object_put(rq);

	content = json_object_object_get(rp, "content");
	if (!content) {
		fprintf(stderr, "Login failed: no content\n");
		return ;
	}

	error = json_object_object_get(content, "error");
	if (error) {
		fprintf(stderr, "Login failed\n");
		return ;
	}

	sid = json_object_object_get(content, "session_id");

	if (session_id) {
		free(session_id);
		session_id = NULL;
	}

	session_id = strdup(json_object_get_string(sid));

	printf("Session id: %s\n", session_id);

	json_object_put(rp);
}

const char *ttrss_get_headline_content(struct headline *h)
{
	struct json_object *rp, *rq, *content, *array, *item;

	printf("get_headlines %d\n", h->id);

	if (!h->content) {
		rq = create_op("getArticle");
		json_object_object_add(rq, "article_id",
				       json_object_new_int(h->id));
		
		rp = post_json_object(session_url, rq);
		
		json_object_put(rq);
		
		array = json_object_object_get(rp, "content");

		if (!array)
			goto release;

		item = json_object_array_get_idx(array, 0);

		if (!item)
			goto release;

		content = json_object_object_get(item, "content");

		h->content = strdup(json_object_get_string(content));
			
	release:
		json_object_put(rp);
	}
	return h->content;
}

static struct headline **get_headlines(int feed_id)
{
	struct json_object *rp, *rq, *content, *jheadline, *j;
	int i, n;
	struct headline **headlines, *h;

	printf("get_headlines %d\n", feed_id);

	rq = create_op("getHeadlines");
	json_object_object_add(rq, "feed_id", json_object_new_int(feed_id));

	rp = post_json_object(session_url, rq);

	json_object_put(rq);

	content = json_object_object_get(rp, "content");

	if (content) {
		n = json_object_array_length(content);
		headlines = malloc((n+1)*sizeof(struct headline *));
		for (i = 0; i < n; i++) {
			jheadline = json_object_array_get_idx(content, i);

			h = malloc(sizeof(struct headline));

			j = json_object_object_get(jheadline, "id");
			h->id = json_object_get_int(j);

			j = json_object_object_get(jheadline, "title");
			h->title = strdup(json_object_get_string(j));

			j = json_object_object_get(jheadline, "link");
			h->url = strdup(json_object_get_string(j));

			h->excerpt = NULL;
			h->content = NULL;

			j = json_object_object_get(jheadline, "unread");
			h->unread = json_object_get_boolean(j);

			headlines[i] = h;
		}
		headlines[n] = NULL;
	} else {
		headlines = NULL;
	}

	json_object_put(rp);

	printf("get_headlines %d end\n", feed_id);

	return headlines;
}

static int feeds_length(struct feed **list)
{
	int n;

	if (!list)
		return 0;
	
	n = 0;
	while(*list) {
		n++;
		list++;
	}

	return n;
}

static struct feed **feeds_add(struct feed **feeds, struct feed *feed)
{
	int n;
	struct feed **result;

	n = feeds_length(feeds);

	result = malloc((n + 1 + 1) * sizeof(struct feed *));

	if (feeds)
		memcpy(result, feeds, n * sizeof(struct feed *));

	result[n] = feed;
	result[n + 1] = NULL;

	return result;
}

struct feed **ttrss_get_feeds()
{
	struct json_object *rp, *rq, *content, *jfeed, *j;
	int i, n;
	struct feed **feeds, *feed, **tmp;

	printf("ttrss_get_feeds\n");

	rq = create_op("getFeeds");

	rp = post_json_object(session_url, rq);
	json_object_put(rq);

	content = json_object_object_get(rp, "content");

	if (content) {
		n = json_object_array_length(content);

		feeds = NULL;
		for (i = 0; i < n; i++) {
			jfeed = json_object_array_get_idx(content, i);

			feed = malloc(sizeof(struct feed));

			j = json_object_object_get(jfeed, "title");
			feed->title = strdup(json_object_get_string(j));

			j = json_object_object_get(jfeed, "feed_url");
			feed->url = strdup(json_object_get_string(j));

			j = json_object_object_get(jfeed, "id");
			feed->id = json_object_get_int(j);

			j = json_object_object_get(jfeed, "unread");
			feed->unread = json_object_get_int(j);

			feed->headlines = NULL;

			tmp = feeds_add(feeds, feed);
			free(feeds);
			feeds = tmp;
		}
	} else {
		feeds = NULL;
	}

	json_object_put(rp);

	printf("ttrss_get_feeds ended\n");

	return feeds;
}

struct headline **ttrss_get_headlines(struct feed *f)
{
	if (!f->headlines)
		f->headlines = get_headlines(f->id);

	return f->headlines;
}

void ttrss_set_article_unread(int id, int unread)
{
	struct json_object *rp, *rq;

	printf("ttrss_set_article_unread %d %d\n", id, unread);

	rq = create_op("updateArticle");
	json_object_object_add(rq, "article_ids", json_object_new_int(id));
	json_object_object_add(rq, "field", json_object_new_int(2));
	json_object_object_add(rq, "mode", json_object_new_int(unread));

	rp = post_json_object(session_url, rq);
	
	json_object_put(rq);
	json_object_put(rp);
}
